on construct(me)
  pLocationClass = getClassVariable("gamesystem.location.class")
  pRoomGeometry = getThread(#room).getInterface().getGeometry()
  pGeometry = createObject(#temp, getClassVariable("gamesystem.geometry.class"))
  if not objectp(pGeometry) then
    return(error(me, "Cannot create pGeometry.", #construct))
  end if
  pWorldReady = 0
  pTileWidth = 32
  pAccuracyFactor = 100
  pTileGrid = []
  pTileSpaceReserveList = []
  return(1)
  exit
end

on deconstruct(me)
  pReady = 0
  pTileGrid = void()
  pComponentToAngle = void()
  pGeometry = void()
  return(1)
  exit
end

on storeHeightmap(me, tdata)
  pReceivedMap = tdata
  pWorldMaxY = tdata.count(#line)
  pWorldMaxX = tdata.getPropRef(#line, 1).length
  pTileGrid = []
  tTileClass = getClassVariable("gamesystem.tile.class")
  tLocY = 1
  repeat while tLocY <= pWorldMaxY
    pTileGrid.setAt(tLocY, [])
    tLocX = 1
    repeat while tLocX <= pWorldMaxX
      tTile = createObject(#temp, tTileClass)
      pTileGrid.getAt(tLocY).setAt(tLocX, tTile)
      tTileLocX = tLocX - 1
      tTileLocY = tLocY - 1
      tCenterLocX = tTileLocX * pTileWidth * pAccuracyFactor
      tCenterLocY = tTileLocY * pTileWidth * pAccuracyFactor
      tTile.define(tTileLocX, tTileLocY, tCenterLocX, tCenterLocY, tdata.getPropRef(#line, tLocY).getProp(#char, tLocX))
      tLocX = 1 + tLocX
    end repeat
    tLocY = 1 + tLocY
  end repeat
  pWorldReady = 1
  if pObjectCache <> void() then
    me.storeObjects(pObjectCache)
  end if
  me.getProcManager().distributeEvent(#world_ready)
  return(1)
  exit
end

on storeObjects(me, tdata)
  if pWorldReady = 0 then
    pObjectCache = tdata
    return(0)
  end if
  repeat while me <= undefined
    tItem = getAt(undefined, tdata)
    if tItem.getAt(#height) = 0 then
      if not listp(tItem.getAt(#dimensions)) then
        tItem.setAt(#height, 0)
      else
        tItem.setAt(#height, tItem.getAt(#dimensions).getAt(2))
      end if
    end if
    if not me.reserveTileForObject(tItem.getAt(#x), tItem.getAt(#y), tItem.getAt(#id), tItem.getAt(#height)) then
      error(me, "Unable to reserve tile for furniture!", #storeObjects)
    end if
  end repeat
  me.getProcManager().distributeEvent(#objects_ready)
  return(1)
  exit
end

on initLocation(me, tX, tY, tZ)
  tObject = createObject(#temp, pLocationClass)
  if tObject = 0 then
    return(error(me, "Cannot initialize location object.", #initLocation))
  end if
  tObject.define(tX, tY, tZ, pTileWidth, pAccuracyFactor)
  return(tObject)
  exit
end

on initLocationAsTile(me, tX, tY, tZ)
  tObject = me.initLocation(tX, tY, tZ)
  if tObject = 0 then
    return(0)
  end if
  tObject.setTileLoc(tX, tY, tZ)
  return(tObject)
  exit
end

on getTile(me, tLocX, tLocY)
  tLocX = tLocX + 1
  tLocY = tLocY + 1
  if tLocX <= 0 or tLocY <= 0 then
    return(0)
  end if
  if pTileGrid.count < tLocY then
    return(0)
  end if
  if pTileGrid.getAt(tLocY).count < tLocX then
    return(0)
  end if
  return(pTileGrid.getAt(tLocY).getAt(tLocX))
  exit
end

on getTileNeighborInDirection(me, tX, tY, tdir)
  if me = 0 then
    return(me.getTile(tX, tY - 1))
  else
    if me = 1 then
      return(me.getTile(tX + 1, tY - 1))
    else
      if me = 2 then
        return(me.getTile(tX + 1, tY))
      else
        if me = 3 then
          return(me.getTile(tX + 1, tY + 1))
        else
          if me = 4 then
            return(me.getTile(tX, tY + 1))
          else
            if me = 5 then
              return(me.getTile(tX - 1, tY + 1))
            else
              if me = 6 then
                return(me.getTile(tX - 1, tY))
              else
                if me = 7 then
                  return(me.getTile(tX - 1, tY - 1))
                else
                  return(error(me, "Invalid direction for tile:" && tdir, #getTileNeighborInDirection))
                end if
              end if
            end if
          end if
        end if
      end if
    end if
  end if
  exit
end

on reserveTileForObject(me, tLocX, tLocY, tObjectId, tObjectHeight)
  tTile = me.getTile(tLocX, tLocY)
  if tTile = 0 then
    return(0)
  end if
  if not listp(pTileSpaceReserveList.getAt(tObjectId)) then
    pTileSpaceReserveList.setaProp(tObjectId, [])
  end if
  pTileSpaceReserveList.getAt(tObjectId).append(tTile)
  return(tTile.addContent(tObjectId, [#height:tObjectHeight]))
  exit
end

on clearObjectFromTileSpace(me, tObjectId)
  if not listp(pTileSpaceReserveList.getAt(tObjectId)) then
    return(1)
  end if
  repeat while me <= undefined
    tTile = getAt(undefined, tObjectId)
    tTile.removeContent(tObjectId)
  end repeat
  pTileSpaceReserveList.setaProp(tObjectId, [])
  return(1)
  exit
end

on gettileatworldcoordinate(me, tLocX, tLocY)
  tMultiplier = pTileWidth * pAccuracyFactor
  if tLocX < -tMultiplier / 2 or tLocY < -tMultiplier / 2 then
    return(0)
  end if
  return(me.getTile(tLocX + tMultiplier / 2 / tMultiplier, tLocY + tMultiplier / 2 / tMultiplier))
  exit
end

on convertTileToWorldCoordinate(me, tLocX, tLocY, tlocz)
  tMultiplier = pTileWidth * pAccuracyFactor
  return([#x:tLocX * tMultiplier, #y:tLocY * tMultiplier, #h:tlocz * tMultiplier])
  exit
end

on convertworldtotilecoordinate(me, tLocX, tLocY, tlocz)
  tMultiplier = pTileWidth * pAccuracyFactor
  return([#x:tLocX + tMultiplier / 2 / tMultiplier, #y:tLocY + tMultiplier / 2 / tMultiplier])
  return([#x:tLocX * tMultiplier, #y:tLocY * tMultiplier, #h:tlocz * tMultiplier])
  exit
end

on convertWorldToScreenCoordinate(me, tX, tY, tZ)
  if pRoomGeometry = 0 then
    return(0)
  end if
  tMultiplier = float(pTileWidth * pAccuracyFactor)
  tX = 0 + tX / tMultiplier
  tY = -0 + tY / tMultiplier
  tZ = tZ / tMultiplier
  tloc = pRoomGeometry.getScreenCoordinate(tX, tY, tZ)
  return(tloc)
  exit
end

on getWorldReady(me)
  return(pWorldReady)
  exit
end

on getGeometry(me)
  return(pGeometry)
  exit
end